import os
import unittest

import openmdao.api as om

from wisdem.rotorse.blade_cost import StandaloneBladeCost, initialize_omdao_prob
from wisdem.glue_code.gc_LoadInputs import WindTurbineOntologyPython
from wisdem.glue_code.gc_PoseOptimization import PoseOptimization

wisdem_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))
example_dir02 = os.path.join(wisdem_dir, "examples", "02_reference_turbines")  # get path example 02_reference_turbines
example_dir03 = os.path.join(wisdem_dir, "examples", "03_blade")  # get path example 03_blade
fname_opt_options = os.path.join(example_dir02, "analysis_options.yaml")
accuracy = 0


class TestBC(unittest.TestCase):
    def testBladeCostIEA3p4(self):
        fname_modeling_options = os.path.join(example_dir02, "modeling_options.yaml")
        fname_wt_input = os.path.join(example_dir02, "IEA-3p4-130-RWT.yaml")
        wt_initial = WindTurbineOntologyPython(fname_wt_input, fname_modeling_options, fname_opt_options)
        wt_init, modeling_options, opt_options = wt_initial.get_input_data()
        modeling_options["WISDEM"]["RotorSE"]["flag"] = False
        modeling_options["WISDEM"]["DriveSE"]["flag"] = False
        modeling_options["WISDEM"]["TowerSE"]["flag"] = False
        modeling_options["flags"]["nacelle"] = False
        modeling_options["flags"]["tower"] = False
        wt_opt = om.Problem(
            reports=False, model=StandaloneBladeCost(modeling_options=modeling_options, opt_options=opt_options)
        )
        wt_opt.setup(derivatives=False)
        myopt = PoseOptimization(wt_init, modeling_options, opt_options)
        wt_opt = myopt.set_initial(wt_opt, wt_init)
        wt_opt = initialize_omdao_prob(wt_opt, modeling_options, wt_init)
        wt_opt.run_model()

        self.assertAlmostEqual(wt_opt["rc.total_labor_hours"][0], 944.1382619536412, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_non_gating_ct"][0], 126.16553815344044, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_metallic_parts_cost"][0], 4667.410277624349, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_consumable_cost_w_waste"][0], 7200.945145281406, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_mat_cost_w_waste"][0], 69710.2136335282, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_labor"][0], 55733.14452425023, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_utility"][0], 639.1295835528967, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_variable_cost"][0], 126082.0797932814, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_equipment"][0], 3355.4596341174124, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_tooling"][0], 9973.915059419693, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_building"][0], 544.9659865519155, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_maintenance_cost"][0], 3592.5310679862623, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_labor_overhead"][0], 16719.943357275068, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.cost_capital"][0], 7757.665254688263, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_fixed_cost"][0], 41944.49845012431, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_cost"][0], 168026.5782434057, places=accuracy)

    def testBladeCostIEA10(self):
        fname_modeling_options = os.path.join(example_dir02, "modeling_options_iea10.yaml")
        fname_wt_input = os.path.join(example_dir02, "IEA-10-198-RWT.yaml")
        wt_initial = WindTurbineOntologyPython(fname_wt_input, fname_modeling_options, fname_opt_options)
        wt_init, modeling_options, opt_options = wt_initial.get_input_data()
        modeling_options["WISDEM"]["RotorSE"]["flag"] = False
        modeling_options["WISDEM"]["DriveSE"]["direct"] = False
        modeling_options["WISDEM"]["DriveSE"]["flag"] = False
        modeling_options["WISDEM"]["TowerSE"]["flag"] = False
        modeling_options["flags"]["nacelle"] = False
        modeling_options["flags"]["tower"] = False
        wt_opt = om.Problem(
            reports=False, model=StandaloneBladeCost(modeling_options=modeling_options, opt_options=opt_options)
        )
        wt_opt.setup(derivatives=False)
        myopt = PoseOptimization(wt_init, modeling_options, opt_options)
        wt_opt = myopt.set_initial(wt_opt, wt_init)
        wt_opt = initialize_omdao_prob(wt_opt, modeling_options, wt_init)
        wt_opt.run_model()

        self.assertAlmostEqual(wt_opt["rc.total_labor_hours"][0], 2129.6348341745884, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_non_gating_ct"][0], 234.2572952143954, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_metallic_parts_cost"][0], 7621.331066101973, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_consumable_cost_w_waste"][0], 12914.424019126209, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_mat_cost_w_waste"][0], 196017.54695398072, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_labor"][0], 126019.47539432498, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_utility"][0], 2594.780222939065, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_variable_cost"][0], 324631.8025712447, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_equipment"][0], 8694.54851553839, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_tooling"][0], 33998.589552977384, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_building"][0], 1974.8871308758073, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_maintenance_cost"][0], 11287.458291742709, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_labor_overhead"][0], 37805.842618297494, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.cost_capital"][0], 22995.045002171933, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_fixed_cost"][0], 116756.37111160373, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_cost"][0], 441388.17368284846, places=accuracy)

    def testBladeCostIEA15(self):
        fname_modeling_options = os.path.join(example_dir02, "modeling_options.yaml")
        fname_wt_input = os.path.join(example_dir02, "IEA-15-240-RWT.yaml")
        wt_initial = WindTurbineOntologyPython(fname_wt_input, fname_modeling_options, fname_opt_options)
        wt_init, modeling_options, opt_options = wt_initial.get_input_data()
        modeling_options["WISDEM"]["RotorSE"]["flag"] = False
        modeling_options["WISDEM"]["DriveSE"]["direct"] = False
        modeling_options["WISDEM"]["DriveSE"]["flag"] = False
        modeling_options["WISDEM"]["TowerSE"]["flag"] = False
        modeling_options["flags"]["nacelle"] = False
        modeling_options["flags"]["tower"] = False
        wt_opt = om.Problem(
            reports=False, model=StandaloneBladeCost(modeling_options=modeling_options, opt_options=opt_options)
        )
        wt_opt.setup(derivatives=False)
        myopt = PoseOptimization(wt_init, modeling_options, opt_options)
        wt_opt = myopt.set_initial(wt_opt, wt_init)
        wt_opt = initialize_omdao_prob(wt_opt, modeling_options, wt_init)
        wt_opt.run_model()

        self.assertAlmostEqual(wt_opt["rc.total_labor_hours"][0], 3132.3284960121646, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_non_gating_ct"][0], 276.2981847459007, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_metallic_parts_cost"][0], 8885.387764565836, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_consumable_cost_w_waste"][0], 15045.994735062488, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_mat_cost_w_waste"][0], 361309.8493400633, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_labor"][0], 185197.24336411885, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_utility"][0], 3625.2319812726428, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_variable_cost"][0], 550132.3246854548, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_equipment"][0], 25738.8550449551, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_tooling"][0], 32203.96254928255, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_cost_building"][0], 2369.9813509855767, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_maintenance_cost"][0], 18291.62879567592, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_labor_overhead"][0], 55559.17300923565, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.cost_capital"][0], 37624.262124040964, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.blade_fixed_cost"][0], 171787.42549803073, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc.total_blade_cost"][0], 721919.7501834856, places=accuracy)

    def testBladeCostBAR_USC(self):
        fname_modeling_options = os.path.join(example_dir03, "modeling_options.yaml")
        fname_wt_input = os.path.join(example_dir03, "BAR_USC.yaml")
        wt_initial = WindTurbineOntologyPython(fname_wt_input, fname_modeling_options, fname_opt_options)
        wt_init, modeling_options, opt_options = wt_initial.get_input_data()
        modeling_options["WISDEM"]["RotorSE"]["flag"] = False
        modeling_options["WISDEM"]["DriveSE"]["flag"] = False
        modeling_options["WISDEM"]["TowerSE"]["flag"] = False
        modeling_options["flags"]["nacelle"] = False
        modeling_options["flags"]["tower"] = False
        wt_opt = om.Problem(
            reports=False, model=StandaloneBladeCost(modeling_options=modeling_options, opt_options=opt_options)
        )
        wt_opt.setup(derivatives=False)
        myopt = PoseOptimization(wt_init, modeling_options, opt_options)
        wt_opt = myopt.set_initial(wt_opt, wt_init)
        wt_opt = initialize_omdao_prob(wt_opt, modeling_options, wt_init)
        wt_opt.run_model()

        self.assertAlmostEqual(wt_opt["rc_in.total_labor_hours"][0], 1846.2238124083613, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_non_gating_ct"][0], 205.19835375754107, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_metallic_parts_cost"][0], 6482.0815222645215, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_consumable_cost_w_waste"][0], 9158.674076916312, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_blade_mat_cost_w_waste"][0], 280171.87668206485, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_cost_labor"][0], 109195.03984379883, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_cost_utility"][0], 1883.9965967071037, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.blade_variable_cost"][0], 391250.9131225708, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_cost_equipment"][0], 10133.962737209944, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_cost_tooling"][0], 17297.355070523197, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_cost_building"][0], 860.7349645109866, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_maintenance_cost"][0], 7854.043863580873, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_labor_overhead"][0], 32758.51195313965, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.cost_capital"][0], 18773.2628923862, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.blade_fixed_cost"][0], 87677.87148135086, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_in.total_blade_cost"][0], 478928.78460392164, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_labor_hours"][0], 382.8251512106479, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_non_gating_ct"][0], 74.06265954079467, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_metallic_parts_cost"][0], 3428.7667570837384, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_consumable_cost_w_waste"][0], 3055.405204719965, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_blade_mat_cost_w_waste"][0], 30489.793345536695, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_cost_labor"][0], 22573.100807894843, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_cost_utility"][0], 171.07321751444601, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.blade_variable_cost"][0], 53233.96737094598, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_cost_equipment"][0], 1500.199865613391, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_cost_tooling"][0], 2577.4998251456445, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_cost_building"][0], 182.64357621312305, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_maintenance_cost"][0], 1231.6522097244074, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_labor_overhead"][0], 6771.930242368453, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.cost_capital"][0], 2857.0127931824622, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.blade_fixed_cost"][0], 15120.938512247481, places=accuracy)
        self.assertAlmostEqual(wt_opt["rc_out.total_blade_cost"][0], 68354.90588319347, places=accuracy)
        self.assertAlmostEqual(wt_opt["total_bc.total_blade_cost"][0], 565883.6904871151, places=accuracy)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestBC))
    return suite


if __name__ == "__main__":
    result = unittest.TextTestRunner().run(suite())

    if result.wasSuccessful():
        exit(0)
    else:
        exit(1)



import os
import csv
from pathlib import Path

import getdocstrings
from docstring_parser import parse

source_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
allpy = Path(source_dir).rglob("*.py") if os.path.exists(source_dir) else []

parsed_args = [["File", "Class", "Var Name", "Var Type", "Var Units", "Var Desc"]]
parsed_dict = {}

for f in allpy:
    if str(f).find("__init__") >= 0:
        continue
    froot = str(f).replace(str(source_dir), "")

    try:
        with open(f) as fp:
            fgroup = getdocstrings.print_docstrings(fp, print_flag=False)
    except:
        print("Could not parse", str(f))
        continue

    for type_, group in fgroup:
        if type_ != "Class":
            continue

        for node, name, lineno, docstring in group:
            parsed_docs = parse(docstring)
            if not hasattr(parsed_docs, "params"):
                continue
            # print(str(f),len(parsed_docs.params))

            for k in range(len(parsed_docs.params)):
                arg_name = parsed_docs.params[k].arg_name
                arg_description = parsed_docs.params[k].description

                type_name = parsed_docs.params[k].type_name
                if not type_name is None:
                    tok = type_name.split(",")
                    arg_type = tok[0]
                    isize = arg_type.find("[")
                    if isize > 0:
                        arg_type = arg_type[:isize]
                else:
                    arg_type = ""

                arg_units = "" if len(tok) <= 1 else tok[1]
                ibracket0 = arg_units.find("[")
                ibracket1 = arg_units.find("]")
                if ibracket0 >= 0 and ibracket1 >= 0:
                    arg_units = arg_units[(ibracket0 + 1) : ibracket1]
                # print('args',parsed_docs.params[k].args)
                # print('default',parsed_docs.params[k].default)
                # print('is_optional',parsed_docs.params[k].is_optional)
                newentry = [froot, name, arg_name, arg_type, arg_units, arg_description]
                parsed_args.append(newentry)
                parsed_dict[arg_name] = arg_description

# Write out these parsed args for later
# with open('wisdem_docargs.csv','w') as fw:
#    writer = csv.writer(fw)
#    writer.writerows(parsed_args)

# Now read in dumps from WISDEM
wisdem_outputs = ["iea15mw_output.csv", "nrel5mw_output.csv"]
master_guide = [["Variable", "Units", "Description"]]
temp_guide = []
varlist = []
for infile in wisdem_outputs:
    with open(infile, "r") as fread:
        ilist = list(csv.reader(fread, dialect="excel"))
        temp_guide.extend(ilist)

# Crude way of making unique vars
temp_guide.sort(key=lambda r: r[0])
for k in temp_guide:
    if k[0] in varlist:
        continue
    varlist.append(k[0])
    master_guide.append([k[0], k[1], k[3]])

# Now fold in the doc strings
for k in master_guide:
    var_name = k[0].split(".")[-1]
    if var_name in parsed_dict:
        idesc = parsed_dict[var_name]
        k[2] += "" if idesc is None else idesc

# Write out these parsed args for later
with open("variable_guide.csv", "w") as fw:
    writer = csv.writer(fw)
    writer.writerows(master_guide)



import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm

from wisdem.moorpy.helpers import CatenaryError


def nonlinear(XF, ZF, L, Str, Ten, W, nNodes=20, plots=0):
    """
    nonlinear mooring line solver.  The nonlinear module is assumed to be off of the seafloor, and have no bending.  The nonlinear module is called by
    putting a tension-strain file in the place of linestiffness (EA).  Unlike the catenary solver no iterative method is needed, but this function is heavily
    borrows from the catenary function.

    Parameters
    ----------
    XF : float
        Horizontal distance from end 1 to end 2 [m]
    ZF : float
        Vertical distance from end 1 to end 2 [m] (positive up)
    L  : float
        Unstretched length of line [m]
    Str : dictionary
        vector of strains from the tension-strain input file [-]
    Ten : dictionary
        vector of  of tensions from the tension-strain input file [N]
    W  : float
        Weight of line in fluid per unit length [N/m]
    nNodes : int, optional
        Number of nodes to describe the line
    plots  : int, optional
        1: plot output, 0: don't


    Returns
    -------
    : tuple
        (end 1 horizontal tension, end 1 vertical tension, end 2 horizontal tension, end 2 vertical tension, info dictionary) [N] (positive up)

    """
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # The following is originally from the catenary.py function but applies to this one aswell
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # make info dict to contain any additional outputs
    info = dict(error=False)

    # make some arrays if needed for plotting each node
    if plots > 0:
        s = np.linspace(
            0, L, nNodes
        )  #  Unstretched arc distance along line from anchor to each node where the line position and tension can be output (meters)
        Xs = np.zeros(nNodes)  #  Horizontal locations of each line node relative to the anchor (meters)
        Zs = np.zeros(nNodes)  #  Vertical   locations of each line node relative to the anchor (meters)
        Te = np.zeros(nNodes)  #  Effective line tensions at each node (N)

    # flip line in the solver if it is buoyant
    if W < 0:
        W = -W
        ZF = -ZF
        CB = -10000.0  # <<< TODO: set this to the distance to sea surface <<<
        flipFlag = True
    else:
        flipFlag = False

    # reverse line in the solver if end A is above end B
    if ZF < 0:
        ZF = -ZF
        reverseFlag = True
    else:
        reverseFlag = False

    # ensure the input variables are realistic
    if XF < 0.0:
        raise CatenaryError("XF is negative!")
    if L <= 0.0:
        breakpoint()
        raise CatenaryError("L is zero or negative!")

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #    New Code
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    # Determine the stretched length of the nonlinear element based on the horizontal and vertical fairlead excursions
    str_L = np.sqrt(XF * XF + ZF * ZF)

    # Based on the stretched length and the horizontal fairlead excursions define cos/sin and products for transformation matrix
    # Define cosine and sine
    c = XF / str_L
    s = ZF / str_L
    # Define all the products of sine and cosine
    c2 = c * c
    s2 = s * s
    cs = c * s

    # Determine the strain in the segment with stretched length and original length
    str_seg = str_L / L - 1

    # Make the input dictionarys for str and ten a vector of floats
    Str = [float(x) for x in Str]
    Ten = [float(x) for x in Ten]

    # Strain in the line cant be less than zero (cant push a rope so to speak)
    if str_seg < 0:
        str_seg = 0

    ten_seg = np.interp(str_seg, Str, Ten)
    # small change is strain for a finite difference
    d_str = 1e-5
    # finite difference to take derivative of ten-strain curve (dT/dstr = EA)
    EA_est = (np.interp(str_seg + d_str, Str, Ten) - np.interp(str_seg, Str, Ten)) / d_str

    # Breakdown the horizontal components of the tension
    FxA = ten_seg * c
    FxB = -FxA

    # Breakdown the vertical components of the tension and subtract the weight of the nonlinear element
    FzA = ten_seg * s - 0.5 * W * L
    FzB = -ten_seg * s - 0.5 * W * L

    # Tension at both ends of the line
    TA = np.sqrt(FxA * FxA + FzA * FzA)
    TB = np.sqrt(FxB * FxB + FzB * FzB)

    # Estimate Stiffness Numerically (Potentially find something better)
    # could also be hard to do because the input tension strain files are arbitrary

    # Stiffness matrcies for the ends of the line (In this case since we already assume the element can only deform axially
    # this is essentially just a truss element with the nonlinear stiffness that we calculated
    # Ka = (EA_est/L)*np.array([[c2, cs],[cs, s2]])
    # Kb = (EA_est/L)*np.array([[c2, cs],[cs, s2]])
    # Kab = -(EA_est/L)*np.array([[c2, cs],[cs, s2]])

    Ka = np.array(
        [
            [
                c2 * (EA_est / str_L) + s2 * ten_seg / (L * (1 + str_seg)),
                cs * (EA_est / str_L) - cs * ten_seg / (L * (1 + str_seg)),
            ],
            [
                cs * (EA_est / str_L) - cs * ten_seg / (L * (1 + str_seg)),
                s2 * (EA_est / str_L) + c2 * ten_seg / (L * (1 + str_seg)),
            ],
        ]
    )
    Kb = Ka
    Kab = -Ka

    # Assign values to info
    # Fairlead forces
    info["HF"] = -FxB
    info["VF"] = -FzB
    info["HA"] = FxA
    info["VA"] = FzA
    # Line stiffnesses
    info["stiffnessA"] = Ka
    info["stiffnessB"] = Kb
    info["stiffnessAB"] = Kab
    # Length on the bottom is zero as we assumed earlier
    info["LBot"] = 0
    # self.info = info
    # For plotting (assumed straight line so no issue and tension varies linearly
    info["X"] = np.linspace(0, XF, nNodes)
    info["Z"] = np.linspace(0, ZF, nNodes)
    info["s"] = np.linspace(0, str_L, nNodes)
    info["Te"] = np.linspace(TA, TB, nNodes)

    print(info["stiffnessA"])
    print(info["stiffnessB"])
    print(info["stiffnessAB"])

    return (FxA, FzA, FxB, FzB, info)
